@@ -781,7 +781,21 @@ itemToReference lang bibtex = bib $ do
   -- hyphenation:
   let defaultHyphenation = case lang of
                                 Lang x y -> x ++ "-" ++ y
-  hyphenation <- ((toLocale . map toLower) <$> getRawField "hyphenation")
+  let getLangId = do
+           langid <- (trim . map toLower) <$> getRawField "langid"
+           idopts <- (trim . map toLower) <$>
+                         getRawField "langidopts" <|> return ""
+           case (langid, idopts) of
+                ("english","variant=british")    -> return "british"
+                ("english","variant=american")   -> return "american"
+                ("english","variant=us")         -> return "american"
+                ("english","variant=usmax")      -> return "american"
+                ("english","variant=uk")         -> return "british"
+                ("english","varant=australian")  -> return "australian"
+                ("english","variant=newzealand") -> return "newzealand"
+                (x,_)                            -> return x
+  hyphenation <- ((toLocale . map toLower) <$>
+                   (getLangId <|> getRawField "hyphenation"))
                 <|> return mempty
 
   -- authors:
@@ -859,6 +873,7 @@ itemToReference lang bibtex = bib $ do
                <|> if subtitle' /= mempty
                       then getShortTitle False "title"
                       else getShortTitle True  "title"
+               <|> return mempty
 
   eventTitle' <- getTitle "eventtitle" <|> return mempty
   origTitle' <- getTitle "origtitle" <|> return mempty
