@@ -82,17 +82,17 @@ disambCitations s bibs cs groups
                     then map (uncurry $ reEvaluate s reEval) $ zip refs groups
                     else groups
 
-      withYearS = if hasYSuffOpt
+      withYearS = addNames $
+                  if hasYSuffOpt
                   then map (mapCitationGroup $ setYearSuffCollision hasNamesOpt needYSuff) $ reEvaluated
                   else rmYearSuff $ reEvaluated
 
-      yearSuffs = when_ hasYSuffOpt . generateYearSuffix bibs . query getYearSuffixes $ withYearS
+      yearSuffs = when_ hasYSuffOpt . generateYearSuffix bibs . concatMap getYearSuffixes $ withYearS
 
       addNames  = proc (updateContrib givenNames newNames newGName)
       processed = if hasYSuffOpt
-                  then proc (updateYearSuffixes yearSuffs) .
-                       addNames $ withYearS
-                  else addNames $ withYearS
+                  then proc (updateYearSuffixes yearSuffs) $ withYearS
+                  else withYearS
 
       citOutput = if disOpts /= [] then processed else reEvaluated
 
@@ -265,7 +265,7 @@ generateYearSuffix refs
       -- sort clashing cites using their position in the sorted bibliography
       getFst . map sort' . map (filter ((/=) 0 . snd)) . map (map getP) .
       -- group clashing cites
-      getFst . map nub . groupBy (\a b -> snd a == snd b) . sort' . filter ((/=) [] . snd)
+      getFst . filter (\grp -> length grp >= 2) . map nub . groupBy (\a b -> snd a == snd b) . sort' . filter ((/=) [] . snd)
     where
       sort'  :: (Ord a, Ord b) => [(a,b)] -> [(a,b)]
       sort'  = sortBy (comparing snd)
@@ -296,10 +296,20 @@ updateYearSuffixes yss o
                              _      -> ONull
  | otherwise             = o
 
-getYearSuffixes :: Output -> [(String,[Output])]
-getYearSuffixes o
-    | OYearSuf _ k c _ <- o = [(k,c)]
-    | otherwise             = []
+getYearSuffixes :: CitationGroup -> [(String,[Output])]
+getYearSuffixes (CG _ _ _ d) = map go d
+  where go (c,x) = (citeId c, relevant False [x])
+        relevant :: Bool -> [Output] -> [Output] -- bool is true if has contrib
+        -- we're only interested in OContrib and OYear, unless there is no OContrib
+        relevant c (Output xs _ : rest) = relevant c xs ++ relevant c rest
+        relevant c (OYear n _ _ : rest) = OStr n emptyFormatting : relevant c rest
+        relevant False (OStr s _    : rest) = OStr s emptyFormatting : relevant False rest
+        relevant False (OSpace      : rest) = OSpace : relevant False rest
+        relevant False (OPan ils    : rest) = OPan ils : relevant False rest
+        relevant _ (OContrib _ _ v _ _ : rest ) = relevant False v ++ relevant True rest
+        relevant c (OName _ v _ _ : rest ) = relevant c v ++ relevant c rest
+        relevant c (_           : rest) = relevant c rest
+        relevant _ []                   = []
 
 rmYearSuff :: [CitationGroup] -> [CitationGroup]
 rmYearSuff = proc rmYS
@@ -346,7 +356,8 @@ hasDuplicates :: Eq a => [a] -> Bool
 hasDuplicates = or . same
 
 allTheSame :: Eq a => [a] -> Bool
-allTheSame = and . same
+allTheSame []     = True
+allTheSame (x:xs) = all (== x) xs
 
 -- | Add the year suffix to the year. Needed for disambiguation.
 addYearSuffix :: Output -> Output
