@@ -1,6 +1,7 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving, PatternGuards, OverloadedStrings,
   DeriveDataTypeable, ExistentialQuantification, FlexibleInstances,
-  ScopedTypeVariables, GeneralizedNewtypeDeriving #-}
+  ScopedTypeVariables, GeneralizedNewtypeDeriving, IncoherentInstances,
+  DeriveGeneric #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.CSL.Reference
@@ -17,9 +18,10 @@
 
 module Text.CSL.Reference where
 
-import Data.List  ( elemIndex, isPrefixOf )
+import Data.List  ( elemIndex, isPrefixOf, intercalate )
 import Data.Maybe ( fromMaybe             )
-import Data.Generics
+import Data.Generics hiding (Generic)
+import GHC.Generics (Generic)
 import Data.Monoid
 import Data.Aeson hiding (Value)
 import qualified Data.Aeson as Aeson
@@ -29,14 +31,14 @@ import qualified Data.Text as T
 import qualified Data.Vector as V
 import Data.Char (toLower, isUpper, isLower, isDigit)
 import Text.CSL.Style hiding (Number)
-import Text.CSL.Util (parseString, parseBool, safeRead, readNum,
+import Text.CSL.Util (parseString, parseInt, parseBool, safeRead, readNum,
                       inlinesToString, capitalize, camelize)
 import Text.Pandoc (Inline(Str,Space))
 import Data.List.Split (wordsBy)
 import Data.String
 
 newtype Literal = Literal { unLiteral :: String }
-  deriving ( Show, Read, Eq, Data, Typeable, Monoid )
+  deriving ( Show, Read, Eq, Data, Typeable, Monoid, Generic )
 
 instance FromJSON Literal where
   parseJSON v             = Literal `fmap` parseString v
@@ -82,7 +84,7 @@ isValueSet val
     | Just _ <- fromValue val :: Maybe Empty     = True
     | otherwise = False
 
-data Empty = Empty deriving ( Typeable, Data )
+data Empty = Empty deriving ( Typeable, Data, Generic )
 
 data Agent
     = Agent { givenName       :: [Formatted]
@@ -93,7 +95,7 @@ data Agent
             , literal         ::  Formatted
             , commaSuffix     ::  Bool
             }
-      deriving ( Show, Read, Eq, Typeable, Data )
+      deriving ( Show, Read, Eq, Typeable, Data, Generic )
 
 instance FromJSON Agent where
   parseJSON (Object v) = Agent <$>
@@ -108,7 +110,8 @@ instance FromJSON Agent where
 
 instance ToJSON Agent where
   toJSON agent = object' $ [
-      "given" .= givenName agent
+      "given" .= Formatted (intercalate [Space] $ map unFormatted
+                                                $ givenName agent)
     , "dropping-particle" .= droppingPart agent
     , "non-dropping-particle" .= nonDroppingPart agent
     , "family" .= familyName agent
@@ -121,9 +124,8 @@ instance FromJSON [Agent] where
   parseJSON (Object v) = (:[]) `fmap` parseJSON (Object v)
   parseJSON _ = fail "Could not parse [Agent]"
 
-instance ToJSON [Agent] where
-  toJSON [x] = toJSON x
-  toJSON xs  = Array (V.fromList $ map toJSON xs)
+-- instance ToJSON [Agent] where
+--  toJSON xs  = Array (V.fromList $ map toJSON xs)
 
 data RefDate =
     RefDate { year   :: Literal
@@ -132,7 +134,7 @@ data RefDate =
             , day    :: Literal
             , other  :: Literal
             , circa  :: Bool
-            } deriving ( Show, Read, Eq, Typeable, Data )
+            } deriving ( Show, Read, Eq, Typeable, Data, Generic )
 
 instance FromJSON RefDate where
   parseJSON (Array v) =
@@ -154,6 +156,7 @@ instance FromJSON RefDate where
               ((v .: "circa" >>= parseBool) <|> pure False)
   parseJSON _ = fail "Could not parse RefDate"
 
+{-
 instance ToJSON RefDate where
   toJSON refdate = object' $ [
       "year" .= year refdate
@@ -162,6 +165,7 @@ instance ToJSON RefDate where
     , "day" .= day refdate
     , "other" .= other refdate ] ++
     [ "circa" .= circa refdate | circa refdate ]
+-}
 
 instance FromJSON [RefDate] where
   parseJSON (Array xs) = mapM parseJSON $ V.toList xs
@@ -174,9 +178,28 @@ instance FromJSON [RefDate] where
          _               -> (:[]) `fmap` parseJSON (Object v)
   parseJSON x          = parseJSON x >>= mkRefDate
 
+toDatePart :: RefDate -> [Int]
+toDatePart refdate =
+    case (safeRead (unLiteral $ year refdate),
+          safeRead (unLiteral $ month refdate),
+          safeRead (unLiteral $ day refdate)) of
+         (Just (y :: Int), Just (m :: Int), Just (d :: Int))
+                                     -> [y, m, d]
+         (Just y, Just m, Nothing)   -> [y, m]
+         (Just y, Nothing, Nothing)  -> [y]
+         _                           -> []
+
 instance ToJSON [RefDate] where
-  toJSON [x] = toJSON x
-  toJSON xs  = Array (V.fromList $ map toJSON xs)
+  toJSON [] = Array V.empty
+  toJSON xs = object' $
+    case filter (not . null) (map toDatePart xs) of
+         []  -> ["literal" .= intercalate "; " (map (unLiteral . other) xs)]
+         dps -> (["date-parts" .= dps ] ++
+                 ["circa" .= (1 :: Int) | or (map circa xs)] ++
+                 ["season" .= s | s <- map season xs, s /= mempty])
+
+-- instance ToJSON [RefDate]
+-- toJSON xs  = Array (V.fromList $ map toJSON xs)
 
 setCirca :: Bool -> RefDate -> RefDate
 setCirca circa' rd = rd{ circa = circa' }
@@ -223,7 +246,7 @@ data RefType
     | Thesis
     | Treaty
     | Webpage
-      deriving ( Read, Eq, Typeable, Data )
+      deriving ( Read, Eq, Typeable, Data, Generic )
 
 instance Show RefType where
     show = map toLower . formatField . showConstr . toConstr
@@ -243,12 +266,10 @@ instance ToJSON RefType where
          uncapitalize (x:xs) = toLower x : xs
          uncapitalize []     = []
 
-newtype CNum = CNum { unCNum :: Int } deriving ( Show, Read, Eq, Num, Typeable, Data )
+newtype CNum = CNum { unCNum :: Int } deriving ( Show, Read, Eq, Num, Typeable, Data, Generic )
 
 instance FromJSON CNum where
-  parseJSON x = case fromJSON x of
-                     Success n -> return $ CNum n
-                     _         -> fail "Could not parse CNum"
+  parseJSON x = CNum `fmap` parseInt x
 
 instance ToJSON CNum where
   toJSON (CNum n) = toJSON n
@@ -335,7 +356,7 @@ data Reference =
     , citationNumber           :: CNum
     , firstReferenceNoteNumber :: Int
     , citationLabel            :: Literal
-    } deriving ( Eq, Show, Read, Typeable, Data )
+    } deriving ( Eq, Show, Read, Typeable, Data, Generic )
 
 instance FromJSON Reference where
   parseJSON (Object v) = Reference <$>
@@ -412,7 +433,7 @@ instance FromJSON Reference where
        v .:? "categories" .!= [] <*>
        v .:? "language" .!= "" <*>
        v .:? "citation-number" .!= CNum 0 <*>
-       v .:? "first-reference-note-number" .!= 1 <*>
+       ((v .: "first-reference-note-number" >>= parseInt) <|> return 1) <*>
        v .:? "citation-label" .!= ""
   parseJSON _ = fail "Could not parse Reference"
 
