@@ -4,6 +4,7 @@ import System.Process
 import System.Exit
 import System.Directory
 import System.FilePath
+import Data.Maybe (fromMaybe)
 import Text.Printf
 import System.IO
 import Data.Monoid (mempty)
@@ -11,20 +12,20 @@ import System.IO.Temp (withSystemTempDirectory)
 import System.Process (rawSystem)
 import Text.Printf
 import qualified Data.Aeson as Aeson
-import Data.Aeson.Encode.Pretty
 import Text.Pandoc.Definition
 import qualified Data.ByteString.Lazy.Char8 as BL
 import qualified Data.ByteString.Char8 as B
 import qualified Text.Pandoc.UTF8 as UTF8
 import Text.Pandoc.Shared (normalize)
 import Text.Pandoc.Process (pipeProcess)
-import qualified Data.Yaml as Yaml (decode)
+import qualified Data.Yaml as Yaml
 import Text.Pandoc (writeNative, writeHtmlString, readNative, def)
 import Text.CSL.Pandoc (processCites')
 
 main = do
   citeprocTests <- mapM testCase ["chicago-author-date", "ieee", "mhra",
-                                  "number-of-volumes", "no-author"]
+                                  "number-of-volumes", "no-author", "issue7",
+                                  "issue13", "issue14"]
   fs <- filter (\f -> takeExtension f `elem` [".bibtex",".biblatex"])
            `fmap` getDirectoryContents "tests/biblio2yaml"
   biblio2yamlTests <- mapM biblio2yamlTest fs
@@ -55,16 +56,28 @@ testCase csl = do
   hPutStr stderr $ "[" ++ csl ++ ".in.json] "
   indataNative <- readFile $ "tests/" ++ csl ++ ".in.native"
   expectedNative <- readFile $ "tests/" ++ csl ++ ".expected.native"
-  let inDoc = read indataNative
-  outDoc <- normalize `fmap` processCites' inDoc
+  let jsonIn = Aeson.encode $ (read indataNative :: Pandoc)
   let expectedDoc = normalize $ read expectedNative
-  if outDoc == expectedDoc
-     then err "PASSED" >> return Passed
+
+  (ec, jsonOut, errout) <- pipeProcess
+                     (Just [("LANG","en_US.UTF-8"),("HOME",".")])
+                     "dist/build/pandoc-citeproc/pandoc-citeproc"
+                     [] jsonIn
+  if ec == ExitSuccess
+     then do
+       let outDoc = normalize $ fromMaybe mempty $ Aeson.decode $ jsonOut
+       if outDoc == expectedDoc
+          then err "PASSED" >> return Passed
+          else do
+             err $ "FAILED"
+             showDiff (UTF8.fromStringLazy $ writeNative def expectedDoc)
+                      (UTF8.fromStringLazy $ writeNative def outDoc)
+             return Failed
      else do
-        err $ "FAILED"
-        showDiff (UTF8.fromStringLazy $ writeNative def expectedDoc)
-                 (UTF8.fromStringLazy $ writeNative def outDoc)
-        return Failed
+       err "ERROR"
+       err $ "Error status " ++ show ec
+       err $ UTF8.toStringLazy errout
+       return Errored
 
 showDiff :: BL.ByteString -> BL.ByteString -> IO ()
 showDiff expected' result' =
@@ -89,22 +102,16 @@ biblio2yamlTest fp = do
   let expected = BL.unlines yamllines
   (ec, result, errout) <- pipeProcess
                      (Just [("LANG","en_US.UTF-8"),("HOME",".")])
-                     "dist/build/biblio2yaml/biblio2yaml"
-                     ["-f", drop 1 $ takeExtension fp] bib
+                     "dist/build/pandoc-citeproc/pandoc-citeproc"
+                     ["bib2yaml", "-f", drop 1 $ takeExtension fp] bib
   if ec == ExitSuccess
      then do
        let expectedDoc :: Maybe Aeson.Value
            expectedDoc = Yaml.decode $ B.concat $ BL.toChunks expected
        let resultDoc   :: Maybe Aeson.Value
            resultDoc   = Yaml.decode $ B.concat $ BL.toChunks result
-       let result' = maybe BL.empty
-                     ( encodePretty' Config{ confIndent = 2
-                                           , confCompare = compare } )
-                     resultDoc
-       let expected' = maybe BL.empty
-                     ( encodePretty' Config{ confIndent = 2
-                                           , confCompare = compare } )
-                     expectedDoc
+       let result'   = BL.fromChunks [Yaml.encode resultDoc]
+       let expected' = BL.fromChunks [Yaml.encode expectedDoc]
        if expected' == result'
           then err "PASSED" >> return Passed
           else do
